package com.usky.issue.service.impl;


import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.issue.domain.YtDeviceStatus;
import com.usky.issue.mapper.YtDeviceStatusMapper;
import com.usky.issue.service.YtDeviceStatusService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.issue.service.vo.YtDeviceStatusRequestVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 离线设备VIEW 服务实现类
 * </p>
 *
 * @author fu
 * @since 2023-11-23
 */
@Service
public class YtDeviceStatusServiceImpl extends AbstractCrudService<YtDeviceStatusMapper, YtDeviceStatus> implements YtDeviceStatusService {
    @Autowired
    private YtDeviceStatusMapper ytDeviceStatusMapper;

    @Override
    public CommonPage<YtDeviceStatus> page(YtDeviceStatusRequestVO requestVO) {

        String deviceType = requestVO.getDeviceType();
        int pageCurrent = requestVO.getCurrent();
        int pageSize = requestVO.getSize();

        List<YtDeviceStatus> list = ytDeviceStatusMapper.getDeviceStatusType(deviceType, pageCurrent, pageSize);
        // 计算总记录数
        int total = list.size();

        // 计算总页数
        int current = (total + pageSize - 1) / pageSize;

        // 计算当前页的起始索引
        int startIndex = (pageCurrent - 1) * pageSize;

        //对传入类型进行判断筛选
        List<YtDeviceStatus> filteredList = list.stream()
                .filter(device -> device.getDeviceType().equals(deviceType))
                .collect(Collectors.toList());

        // 对记录进行分页操作
        List<YtDeviceStatus> records = filteredList.stream()
                .skip(startIndex)//跳过startIndex个元素，从startIndex+1开始
                .limit(pageSize)//限制显示大小，只处理pageSize个元素
                .collect(Collectors.toList());//将流中的元素收集到List<YtDeviceStatus> == 获取当前页记录

        return new CommonPage<>(records, total, pageSize, pageCurrent);
    }

    /*
    导出离线数据查询结果
     */
    @Override
    public List<YtDeviceStatus> export() {
        return ytDeviceStatusMapper.exportData();
    }

    /*
    根据传入单位编号查询所有设备信息
    */
    @Override
    public List<Map<String, Object>> deviceList(YtDeviceStatusRequestVO requestVO) {
        String companyCode = requestVO.getCompanyCode();
        // 查询各类型设备的总数以及在线、离线设备数量
        List<Map<String, Object>> statusCounts = ytDeviceStatusMapper.deviceList(companyCode);
        Map<String, Map<String, Integer>> statusMap = new HashMap<>();
        for (Map<String, Object> statusCount : statusCounts) {
            String deviceType = (String) statusCount.get("deviceType");
            Integer onlineCount = ((Number) statusCount.get("onlineCount")).intValue();
            Integer offlineCount = ((Number) statusCount.get("offlineCount")).intValue();
            statusMap.computeIfAbsent(deviceType, k -> new HashMap<>())
                    .put("onlineCount", onlineCount);
            statusMap.computeIfAbsent(deviceType, k -> new HashMap<>())
                    .put("offlineCount", offlineCount);
            int totalCount = ((Number) statusCount.get("deviceTypeCounts")).intValue();
            statusMap.computeIfAbsent(deviceType, k -> new HashMap<>()).put("deviceTypeCounts", totalCount);
        }
        // 统计设备总数和在线、离线设备数
        Map<String, Integer> deviceTypeCounts = new HashMap<>();
        Map<String, Integer> onlineDeviceCount = new HashMap<>();
        Map<String, Integer> offlineDeviceCount = new HashMap<>();

        // 设置设备状态并统计设备数量
        List<Map<String, Object>> deviceList = ytDeviceStatusMapper.deviceList(companyCode);
        for (Map<String, Object> device : deviceList) {
            String deviceType = (String) device.get("deviceType");
            deviceTypeCounts.merge(deviceType, (Integer) statusMap.getOrDefault(deviceType, Collections.emptyMap()).getOrDefault("deviceTypeCounts", 0), Integer::sum);

            Integer onlineCount = statusMap.getOrDefault(deviceType, Collections.emptyMap())
                    .getOrDefault("onlineCount", 0);
            Integer offlineCount = statusMap.getOrDefault(deviceType, Collections.emptyMap())
                    .getOrDefault("offlineCount", 0);
            Integer currentOnlineCount = onlineDeviceCount.getOrDefault(deviceType, 0);
            Integer currentOfflineCount = offlineDeviceCount.getOrDefault(deviceType, 0);
            onlineDeviceCount.put(deviceType, currentOnlineCount + onlineCount);
            offlineDeviceCount.put(deviceType, currentOfflineCount + offlineCount);
        }

        // 添加设备类型、设备总数和在线、离线设备数到结果列表
        List<Map<String, Object>> deviceStatusList = new ArrayList<>();
        for (String deviceType : deviceTypeCounts.keySet()) {
            Map<String, Object> deviceInfo = new HashMap<>();
            deviceInfo.put("deviceType", deviceType);
            deviceInfo.put("deviceTypeCounts", deviceTypeCounts.getOrDefault(deviceType, 0));
            deviceInfo.put("onlineDeviceCount", onlineDeviceCount.getOrDefault(deviceType, 0));
            deviceInfo.put("offlineDeviceCount", offlineDeviceCount.getOrDefault(deviceType, 0));
            deviceStatusList.add(deviceInfo);
        }

        return deviceStatusList;
    }

}